/* ***********************************************************
Johannes Philippi, Jan.2007
johannesphilippi[a t]web[d o t]de

In dieser Datei werden definiert:
- Daten und Control Register/Ports/Pins fr das Diplay
- Positionen auf dem Display 
	(p11 = 1Zeile 1.Zeichen, p216 = 2Zeile 16.Zeichen)
	
ACHTUNG!! Zusammen mit dieser Datei muss auch die libm-Bibliothek 
eingbunden werden [ wegen der funktion dtostr(...) ].
*********************************************************** */

#include <avr/io.h>           	//names of bytes and bits of I/O registers
#include <inttypes.h>
#include <stdlib.h>
//#include "delaymsCAN128.c" 	//must eventually be included

// Adapt to your HW!!
// 2x16 LCD connected to PORTA
#define LCD_DDR    	DDRA  	//port to which the display is connected
#define LCD_OUT    	PORTA
#define LCD_IN     	PINA

//The control Signals on PORTC are as follows:
#define LCD_CTRL_DDR	DDRC
#define LCD_CTRL_PORT	PORTC
#define LCD_RS     		7     	//pin for RS
#define LCD_E      		6     	//pin for Enable
//the read/write signal is connected to GND =>only write to LCD

//erste Zeile
#define p11 0x80
#define p12 0x81
#define p13 0x82
#define p14 0x83
#define p15 0x84
#define p16 0x85
#define p17 0x86
#define p18 0x87
#define p19 0x88
#define p110 0x89
#define p111 0x8A
#define p112 0x8B
#define p113 0x8C
#define p114 0x8D
#define p115 0x8E
#define p116 0x8F

//zweite Zeile
#define p21 0xC0
#define p22 0xC1
#define p23 0xC2
#define p24 0xC3
#define p25 0xC4
#define p26 0xC5
#define p27 0xC6
#define p28 0xC7
#define p29 0xC8
#define p210 0xC9
#define p211 0xCA
#define p212 0xCB
#define p213 0xCC
#define p214 0xCD
#define p215 0xCE
#define p216 0xCF

/******************************************************************
function prototypes
******************************************************************/
void lcd_init ();
void lcd_write (uint8_t data, uint8_t type);
void lcd_write_string (uint8_t *data);
void lcd_write_number (float fl);
void lcd_clear ();

/******************************************************************
lcd_init() does the initialisation of the LC display(driver) as in 
the datasheet "Character Type Dot Matrix LCD Module" 162A by 
www.tridentdisplays.cu.uk (flowchart on page16). The intervals are 
realized with the nopCycles(k) function.
******************************************************************/
void lcd_init ()
{
  //Timer initilisieren fr delayms-Routine
  TCCR0A = 0x00;
  TCCR0A |= (1<<CS00); 
  TIMSK0 &= ~(1<<TOIE0) & ~(1<<OCIE0A);
  
  DDRC  = 0x3F;	//output pins fr LEDs
  

  // set Port direction register for LCD Data bus
  LCD_DDR = 0xff;
  //set Port direction register for LCD Control bus
  LCD_CTRL_DDR |= (1 << LCD_RS) | (1 << LCD_E);

  delayms(5);
  lcd_write(0x38,0);  	//Function set (8bit mode,2 display lines
     
  //nopCycles(29); 		//ca.39us
  delayms(1);
  
  lcd_write(0x0C,0);  	//Display On

  //nopCycles(29); 		//ca.39us
  delayms(1);

  lcd_clear(0x80,0);  	//Clear Display and Set DD-Ram Adresse = 0

  //nopCycles(1223); 	//ca. 1.53ms
  delayms(3);
  lcd_write(0x06,0);	//entry mode set: increase DD-RAM address at write, no display shift

  return;
//initialization end
}

/******************************************************************
lcd_clear() clears the lcd, sets the cursor to invisible and the 
DD-RAM address to zero.
******************************************************************/
void lcd_clear ()
{
  lcd_write (0x01,0); 	//Clear Display
  //nopCycles(29);		//ca. 39us
  delayms(1);  
  lcd_write (0x80,0);  	//Set DD-Ram Adresse = 0
  return;
}

/******************************************************************
lcd_write(data,type) writes data to the display. The display must 
be initialized in 8bit mode.
Parameters:
type = 0, control command: lcd_write(pxxx,0); sets the LCD cursor position
type = 1, data for display
data = 8bits that are written to the display (data or command)
******************************************************************/
void lcd_write (uint8_t data, uint8_t type)
{
  //set writing signal (RW-Pin = Low)
  //LCD_CTRL_PORT &= ~(1<<LCD_RW); [in my circuit RW=low because RW=GND]
  
  if (type == 0)
    { 	//write to control register
    	LCD_CTRL_PORT &= ~(1<<LCD_RS);  
    }
    else
    {	//write to data register
    	LCD_CTRL_PORT |= (1<<LCD_RS); 
    }
  
  //write 8 bits to LCD
  LCD_OUT = data;
  
  //set and  reset enable bit
  LCD_CTRL_PORT |= (1<<LCD_E); 
  delayms(1);
  LCD_CTRL_PORT &= ~(1<<LCD_E);
  
  //while ((Read_LCD(0)&(1<<BusyBit)) > 0) {}; //Wartet bis LCD wieder bereit

  return;
}

/******************************************************************
lcd_write_string(data,type) writes a string to the display. 
The display must be initialized in 8bit mode.
Parameters:
*data = char-pointer to the string
******************************************************************/
void lcd_write_string (uint8_t *data)
{
uint8_t buchstabe;   
  //set writing signal (RW-Pin = Low)
  //LCD_CTRL_PORT &= ~(1<<LCD_RW); [in my circuit RW=low because RW=GND]
  
  //write to data register
  LCD_CTRL_PORT |= (1<<LCD_RS); 
    
  
  buchstabe = data[0];

  while (buchstabe != '\0')
  {
  	//write 8 bits to LCD
  	LCD_OUT = buchstabe;
  
 	 //set and  reset enable bit
  	LCD_CTRL_PORT |= (1<<LCD_E); 
  	delayms(1);
  	LCD_CTRL_PORT &= ~(1<<LCD_E);
  	
	//increment pointer to next charachter
	data++;
	buchstabe = data[0];

  }
  //while ((Read_LCD(0)&(1<<BusyBit)) > 0) {}; //Wartet bis LCD wieder bereit

  return;
}

/******************************************************************
lcd_write_number(number) writes a number to the display. The number
is converted to a string, which is then printed out to the LCD.
The display must be initialized in 8bit mode.
Parameters:
number = a float number
******************************************************************/
void lcd_write_number (float number)
{
uint8_t str[7];

	//convert number into formatted string with (D)ouble(TOSTR)ing(F)loat -function
	dtostrf(number,3,0,str);		//this function needs the libm.a library included
	lcd_write_string(&str);

}
